@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011-2012 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
+# This code is distributed under the two-clause BSD license.
 
 from django import VERSION as django_version
 from django.core import exceptions
 from django.core import serializers
 from django.db import models as django_models
+from django import template
 from django import test
+from django.template import Template, Context
 from django.utils import unittest
 
 import xworkflows
@@ -363,3 +366,59 @@ class SouthTestCase(test.TestCase):
 
         self.assertEqual(models.MyWorkflow.initial_state.name,
             frozen_field.workflow.initial_state.name)
+
+
+class TemplateTestCase(test.TestCase):
+    """Tests states and transitions behavior in templates."""
+
+    uTrue = unicode(True)
+    uFalse = unicode(False)
+
+    def setUp(self):
+        self.obj = models.MyWorkflowEnabled()
+        self.context = template.Context({'obj': self.obj, 'true': self.uTrue, 'false': self.uFalse})
+
+    def render_fragment(self, fragment):
+        return template.Template(fragment).render(self.context)
+
+    @unittest.skipIf(int(xworkflows.__version__.split('.')[0]) >= 1, "Behaviour changed in xworkflows-1.0.0")
+    def test_state_display_is_title(self):
+        self.assertEqual(self.render_fragment("{{obj.state}}"), self.obj.state.state.title)
+
+    @unittest.skipIf(int(xworkflows.__version__.split('.')[0]) < 1, "Behaviour changed in xworkflows-1.0.0")
+    def test_state_display_is_name(self):
+        self.assertEqual(self.render_fragment("{{obj.state}}"), self.obj.state.state.name)
+
+    def test_state(self):
+        self.assertEqual(self.render_fragment("{{ obj.state.is_foo }}"), self.uTrue)
+        self.assertEqual(self.render_fragment("{% if obj.state == 'foo' %}{{ true }}{% else %}{{ false }}{% endif %}"), self.uTrue)
+
+        self.assertEqual(self.render_fragment("{{ obj.state.is_bar }}"), self.uFalse)
+        self.assertEqual(self.render_fragment("{% if obj.state == 'bar' %}{{ true }}{% else %}{{ false }}{% endif %}"), self.uFalse)
+
+    def test_django_magic(self):
+        """Ensure that ImplementationWrappers have magic django attributes."""
+        self.assertTrue(self.obj.foobar.alters_data)
+        self.assertTrue(self.obj.foobar.do_not_call_in_templates)
+
+    @unittest.skipIf(django_version[:2] >= (1, 4), "foo.do_not_call_in_templates implemented since django>=1.4")
+    def test_transition_hidden(self):
+        """Tests that django (<1.4) will prevent calling the template."""
+
+        self.assertEqual(self.render_fragment("{{ obj.foobar}}"), "")
+        self.assertEqual(self.render_fragment("{{ obj.foobar.is_available }}"), "")
+        self.assertEqual(models.MyWorkflow.states.foo, self.obj.state)
+
+        self.assertEqual(self.render_fragment("{{ obj.bazbar|safe}}"), "")
+        self.assertEqual(self.render_fragment("{{ obj.bazbar.is_available }}"), "")
+        self.assertEqual(models.MyWorkflow.states.foo, self.obj.state)
+
+    @unittest.skipIf(django_version[:2] < (1, 4), "foo.do_not_call_in_templates requires django>=1.4")
+    def test_transaction_attributes(self):
+        self.assertEqual(self.render_fragment("{{ obj.foobar|safe}}"), unicode(self.obj.foobar))
+        self.assertEqual(self.render_fragment("{{ obj.foobar.is_available }}"), self.uTrue)
+        self.assertEqual(models.MyWorkflow.states.foo, self.obj.state)
+
+        self.assertEqual(self.render_fragment("{{ obj.bazbar|safe}}"), unicode(self.obj.bazbar))
+        self.assertEqual(self.render_fragment("{{ obj.bazbar.is_available }}"), self.uFalse)
+        self.assertEqual(models.MyWorkflow.states.foo, self.obj.state)
